@@ -5,7 +5,17 @@
 ##   library.dynam("TMB", pkg, lib)
 ## }
 
-checkMatrixPackageVersion <- function() {
+## Matrix package versions that are likely to break ABI compatibility:
+MVI <- c("1.6-2", "1.7-0")
+## findInterval
+findMVI <- function(v) {
+    sum(MVI <= numeric_version(v))
+}
+sameMVI <- function(x, y) {
+    findMVI(x) == findMVI(y)
+}
+
+checkMatrixPackageVersion <- function(warn=TRUE) {
     ## It is unsafe to use the TMB package with versions of 'Matrix'
     ## other than the one TMB was originally built with.
     file <- paste0(system.file(package="TMB"),"/Matrix-version")
@@ -14,7 +24,7 @@ checkMatrixPackageVersion <- function() {
         writeLines(cur.Matrix.version, con = file)
     }
     TMB.Matrix.version <- readLines(file)
-    if(!identical(TMB.Matrix.version, cur.Matrix.version)) {
+    if(warn && !sameMVI(TMB.Matrix.version, cur.Matrix.version)) {
         warning(
             "Package version inconsistency detected.\n",
             "TMB was built with Matrix version ",
@@ -31,7 +41,12 @@ checkMatrixPackageVersion <- function() {
 
 .onLoad <- function(lib, pkg) {
     library.dynam("TMB", pkg, lib)
-    checkMatrixPackageVersion()
+    checkMatrixPackageVersion(getOption("TMB.check.Matrix", TRUE))
+    ## Select AD framework (CppAD or TMBad) used by TMB::compile
+    tmb.ad.framework <- getOption("tmb.ad.framework", NULL)
+    if (is.null(tmb.ad.framework))
+        tmb.ad.framework <- Sys.getenv("TMB_AD_FRAMEWORK", "CppAD")
+    options("tmb.ad.framework" = tmb.ad.framework)
 }
 
 .onUnload <- function(libpath) {
@@ -47,5 +62,5 @@ checkMatrixPackageVersion <- function() {
 .onAttach <- function(lib, pkg) {
   exfolder <- system.file("examples", package = "TMB")
   dll <- paste0(exfolder, Sys.getenv("R_ARCH"), "/simple", .Platform$dynlib.ext)
-  if(!file.exists(dll)) runExample("simple", dontrun=TRUE)
+  if(!file.exists(dll)) runExample("simple", dontrun=TRUE, eigen.disable.warnings=FALSE)
 }
